@@ -18,7 +18,7 @@ import copy
 from robot import utils
 from robot.errors import DataError
 from robot.variables import GLOBAL_VARIABLES, is_scalar_var
-from robot.output import LOGGER
+from robot.output import LOGGER, Message
 from robot.parsing.settings import Library, Variables, Resource
 
 from .usererrorhandler import UserErrorHandler
@@ -48,16 +48,13 @@ class Namespace:
         self.test = None
         self.uk_handlers = []
         self.variables = _VariableScopes(variables, parent_variables)
-        self.library_search_order = []
         self._imports = imports
-        self._user_keywords = UserLibrary(user_keywords)
-        self._testlibs = {}
-        self._imported_resource_files = ImportCache()
+        self._kw_store = KeywordStore(user_keywords)
         self._imported_variable_files = ImportCache()
 
     @property
     def libraries(self):
-        return list(self._testlibs.itervalues())
+        return self._kw_store.libraries.values()
 
     def handle_imports(self):
         self._import_default_libraries()
@@ -93,11 +90,11 @@ class Namespace:
     def _import_resource(self, import_setting, overwrite=False):
         path = self._resolve_name(import_setting)
         self._validate_not_importing_init_file(path)
-        if overwrite or path not in self._imported_resource_files:
+        if overwrite or path not in self._kw_store.resources:
             resource = IMPORTER.import_resource(path)
             self.variables.set_from_variable_table(resource.variable_table,
                                                    overwrite)
-            self._imported_resource_files[path] \
+            self._kw_store.resources[path] \
                 = UserLibrary(resource.keyword_table.keywords, resource.source)
             self._handle_imports(resource.setting_table.imports)
         else:
@@ -133,11 +130,11 @@ class Namespace:
         name = self._resolve_name(import_setting)
         lib = IMPORTER.import_library(name, import_setting.args,
                                       import_setting.alias, self.variables)
-        if lib.name in self._testlibs:
+        if lib.name in self._kw_store.libraries:
             LOGGER.info("Test library '%s' already imported by suite '%s'"
                         % (lib.name, self.suite.longname))
             return
-        self._testlibs[lib.name] = lib
+        self._kw_store.libraries[lib.name] = lib
         lib.start_suite()
         if self.test:
             lib.start_test()
@@ -173,23 +170,28 @@ class Namespace:
         if name in self._deprecated_libraries:
             self.import_library(self._deprecated_libraries[name])
 
+    def set_search_order(self, new_order):
+        old_order = self._kw_store.search_order
+        self._kw_store.search_order = new_order
+        return old_order
+
     def start_test(self, test):
         self.test = test
         self.variables.start_test()
-        for lib in self._testlibs.values():
+        for lib in self.libraries:
             lib.start_test()
 
     def end_test(self):
         self.test = None
         self.variables.end_test()
         self.uk_handlers = []
-        for lib in self._testlibs.values():
+        for lib in self.libraries:
             lib.end_test()
 
     def end_suite(self):
         self.suite = None
         self.variables.end_suite()
-        for lib in self._testlibs.values():
+        for lib in self.libraries:
             lib.end_suite()
 
     def start_user_keyword(self, handler):
@@ -201,16 +203,11 @@ class Namespace:
         self.uk_handlers.pop()
 
     def get_library_instance(self, libname):
-        try:
-            return self._testlibs[libname.replace(' ', '')].get_instance()
-        except KeyError:
-            raise DataError("No library with name '%s' found." % libname)
+        return self._kw_store.get_library(libname).get_instance()
 
     def get_handler(self, name):
         try:
-            handler = self._get_handler(name)
-            if handler is None:
-                raise DataError("No keyword with name '%s' found." % name)
+            handler = self._kw_store.get_handler(name)
         except DataError, err:
             handler = UserErrorHandler(name, unicode(err))
         self._replace_variables_from_user_handlers(handler)
@@ -220,6 +217,36 @@ class Namespace:
         if hasattr(handler, 'replace_variables'):
             handler.replace_variables(self.variables)
 
+
+class KeywordStore(object):
+
+    def __init__(self, user_keywords):
+        self.user_keywords = UserLibrary(user_keywords)
+        self.libraries = {}
+        self.resources = ImportCache()
+        self.search_order = ()
+
+    def get_library(self, name):
+        try:
+            return self.libraries[name.replace(' ', '')]
+        except KeyError:
+            raise DataError("No library with name '%s' found." % name)
+
+    def get_handler(self, name):
+        handler = self._get_handler(name)
+        if handler is None:
+            self._raise_no_keyword_found(name)
+        return handler
+
+    def _raise_no_keyword_found(self, name):
+        msg = "No keyword with name '%s' found." % name
+        finder = KeywordRecommendationFinder(self.user_keywords,
+                                             self.libraries,
+                                             self.resources)
+        recommendations = finder.recommend_similar_keywords(name)
+        msg = finder.format_recommendations(msg, recommendations)
+        raise DataError(msg)
+
     def _get_handler(self, name):
         handler = None
         if not name:
@@ -274,28 +301,27 @@ class Namespace:
         return None
 
     def _get_handler_from_test_case_file_user_keywords(self, name):
-        if self._user_keywords.has_handler(name):
-            return self._user_keywords.get_handler(name)
+        if self.user_keywords.has_handler(name):
+            return self.user_keywords.get_handler(name)
 
     def _get_handler_from_resource_file_user_keywords(self, name):
-        found = [lib.get_handler(name) for lib
-                 in self._imported_resource_files.values()
+        found = [lib.get_handler(name) for lib in self.resources.values()
                  if lib.has_handler(name)]
         if not found:
             return None
         if len(found) > 1:
-            found = self._get_handler_based_on_library_search_order(found)
+            found = self._get_handler_based_on_search_order(found)
         if len(found) == 1:
             return found[0]
         self._raise_multiple_keywords_found(name, found)
 
     def _get_handler_from_library_keywords(self, name):
-        found = [lib.get_handler(name) for lib in self._testlibs.values()
+        found = [lib.get_handler(name) for lib in self.libraries.values()
                  if lib.has_handler(name)]
         if not found:
             return None
         if len(found) > 1:
-            found = self._get_handler_based_on_library_search_order(found)
+            found = self._get_handler_based_on_search_order(found)
         if len(found) == 2:
             found = self._prefer_process_over_operatingsystem(*found)
         if len(found) == 2:
@@ -304,8 +330,8 @@ class Namespace:
             return found[0]
         self._raise_multiple_keywords_found(name, found)
 
-    def _get_handler_based_on_library_search_order(self, handlers):
-        for libname in self.library_search_order:
+    def _get_handler_based_on_search_order(self, handlers):
+        for libname in self.search_order:
             for handler in handlers:
                 if utils.eq(libname, handler.libname):
                     return [handler]
@@ -320,36 +346,51 @@ class Namespace:
 
     def _filter_stdlib_handler(self, handler1, handler2):
         if handler1.library.orig_name in STDLIB_NAMES:
-            standard, external = handler1, handler2
+            standard, custom = handler1, handler2
         elif handler2.library.orig_name in STDLIB_NAMES:
-            standard, external = handler2, handler1
+            standard, custom = handler2, handler1
         else:
             return [handler1, handler2]
-        if not RUN_KW_REGISTER.is_run_keyword(external.library.orig_name, external.name):
-            LOGGER.warn(
-                "Keyword '%s' found both from a user created test library "
-                "'%s' and Robot Framework standard library '%s'. The user "
-                "created keyword is used. To select explicitly, and to get "
-                "rid of this warning, use either '%s' or '%s'."
-                % (standard.name,
-                   external.library.orig_name, standard.library.orig_name,
-                   external.longname, standard.longname))
-        return [external]
+        if not RUN_KW_REGISTER.is_run_keyword(custom.library.orig_name, custom.name):
+            self._custom_and_standard_keyword_conflict_warning(custom, standard)
+        return [custom]
+
+    def _custom_and_standard_keyword_conflict_warning(self, custom, standard):
+        custom_with_name = standard_with_name = ''
+        if custom.library.name != custom.library.orig_name:
+            custom_with_name = " imported as '%s'" % custom.library.name
+        if standard.library.name != standard.library.orig_name:
+            standard_with_name = " imported as '%s'" % standard.library.name
+        warning = Message("Keyword '%s' found both from a custom test library "
+                          "'%s'%s and a standard library '%s'%s. The custom "
+                          "keyword is used. To select explicitly, and to get "
+                          "rid of this warning, use either '%s' or '%s'."
+                          % (standard.name,
+                             custom.library.orig_name, custom_with_name,
+                             standard.library.orig_name, standard_with_name,
+                             custom.longname, standard.longname), level='WARN')
+        if custom.pre_run_messages:
+            custom.pre_run_messages.append(warning)
+        else:
+            custom.pre_run_messages = [warning]
 
     def _get_explicit_handler(self, name):
-        libname, kwname = name.rsplit('.', 1)
-        # 1) Find matching lib(s)
-        libs = [lib for lib
-                in self._imported_resource_files.values() + self._testlibs.values()
-                if utils.eq(lib.name, libname)]
-        if not libs:
-            return None
-        # 2) Find matching kw from found libs
-        found = [lib.get_handler(kwname) for lib in libs
-                 if lib.has_handler(kwname)]
+        found = []
+        for owner_name, kw_name in self._yield_owner_and_kw_names(name):
+            found.extend(self._find_keywords(owner_name, kw_name))
         if len(found) > 1:
             self._raise_multiple_keywords_found(name, found, implicit=False)
-        return found and found[0] or None
+        return found[0] if found else None
+
+    def _yield_owner_and_kw_names(self, full_name):
+        tokens = full_name.split('.')
+        for i in range(1, len(tokens)):
+            yield '.'.join(tokens[:i]), '.'.join(tokens[i:])
+
+    def _find_keywords(self, owner_name, name):
+        return [owner.get_handler(name)
+                for owner in self.libraries.values() + self.resources.values()
+                if utils.eq(owner.name, owner_name) and owner.has_handler(name)]
 
     def _raise_multiple_keywords_found(self, name, found, implicit=True):
         error = "Multiple keywords with name '%s' found.\n" % name
@@ -360,6 +401,55 @@ class Namespace:
         raise DataError(error)
 
 
+class KeywordRecommendationFinder(object):
+
+    def __init__(self, user_keywords, libraries, resources):
+        self.user_keywords = user_keywords
+        self.libraries = libraries
+        self.resources = resources
+
+    def recommend_similar_keywords(self, name):
+        """Return keyword names similar to `name`."""
+        candidates = self._get_candidates('.' in name)
+        normalizer = lambda name: candidates.get(name, name).lower().replace(
+            '_', ' ')
+        finder = utils.RecommendationFinder(normalizer)
+        return finder.find_recommendations(name, candidates)
+
+    @staticmethod
+    def format_recommendations(msg, recommendations):
+        return utils.RecommendationFinder.format_recommendations(
+            msg, recommendations)
+
+    def _get_candidates(self, use_full_name):
+        names = {}
+        for owner, name in self._get_all_handler_names():
+            full_name = '%s.%s' % (owner, name) if owner else name
+            names[full_name] = full_name if use_full_name else name
+        return names
+
+    def _get_all_handler_names(self):
+        """Return a list of (library name, handler name) tuples.
+
+        For user keywords, library name == None.
+
+        Excludes DeprecatedBuiltIn, DeprecatedOperatingSystem,
+        and Reserved libraries.
+        """
+        excluded = ['DeprecatedBuiltIn', 'DeprecatedOperatingSystem',
+                    'Reserved']
+        handlers = [(None, utils.printable_name(handler_name, True))
+                    for handler_name in self.user_keywords.handlers.keys()]
+        for library in (self.libraries.values() + self.resources.values()):
+            if library.name not in excluded:
+                handlers.extend(
+                    ((library.name,
+                      utils.printable_name(handler_name, code_style=True))
+                     for handler_name in library.handlers))
+        # sort handlers to ensure consistent ordering between Jython and Python
+        return sorted(handlers)
+
+
 class _VariableScopes:
 
     def __init__(self, suite_variables, parent_variables):
